// Tree Gauge Version
// for Tree generating, Gmsh is used to generate the tree
// path for gauging.

Include "ring3d_id.geo";

Dir="ring3d_tg/";

ExtGmsh     = ".pos";
ExtGnuplot  = ".dat";

Group {
  CoreE = Region[ {ECORE} ];
  CoreI = Region[ {ICORE} ];
//  Core = Region[ {CoreE, CoreI} ];
  Core = Region[ {CoreE} ];

  Inds      = Region[{COIL}] ;
  SkinInds  = Region[{SKINCOIL}] ;

  Skin_ECore = Region[ {SKINECORE} ];
  Skin_ICore = Region[ {SKINICORE} ];
//  SkinCore = Region[{Skin_ECore, Skin_ICore}];
  SkinCore = Region[{Skin_ECore}];

  Air  = Region[ AIR ];
  AirInf = Region[ AIRINF ];

  Air  += Region[ {ICORE} ];

  // Surfaces for imposing boundary conditions
  Surf_Inf = Region[ SURF_AIROUT ];

  DomainDummy = Region[ 12345 ];
}

Function {
  DefineConstant[
    Freq = { 50., Min 0, Max 1e3, Step 1,
      Name "Input/21Frequency [Hz]", Highlight "AliceBlue"}
  ];

  vDir[] = ( Vector [Sin[Atan2[Y[],X[]]#1], -Cos[#1],0] ); 
  //#1: register of 1 refer to p22 getdp manual

  js0[Inds] = vDir[] ; // DomainS

  // Material properties
  mu0 = 4.e-7 * Pi ;

  // Use if linear problem
  DefineConstant[
    sigma_coil = { sigma_cu, Label "Conductivity [S/m]",
      Name "Input/4Coil Parameters/5Conductivity", Highlight "AliceBlue"},
    mur_fe = { 2000., Min 100, Max 2000, Step 100,
      Name "Input/42Core relative permeability", Highlight "AliceBlue"},
    sigma_core = { 1e4, Label "Core conductivity [S/m]",
      Name "Input/43Core conductivity", Highlight "AliceBlue"}
  ];

}

Group {
  DefineGroup[
    Domain, DomainCC, DomainC,
    DomainS,
    SkinDomainC,
    Surf_Inf, Surf_FixedMVP
  ] ;
}


Function {
  DefineFunction[
    mu, nu, sigma, rho, js
  ] ;

  DefineConstant[
    Freq, T = 1/Freq, // Fundamental period in s
    II, VV,
    po = "Output 3D/"
  ] ;

}

Group {
  DomainCC = Region[ {Air, AirInf} ];
  DomainC  = Region[ {Core} ];
  SkinDomainC = Region[ {SkinCore} ];

  //--------------------------------------------------------------

  DomainS = Region[ {Inds} ];
  DomainCC += Region[ {DomainS} ];

  //--------------------------------------------------------------

  Domain  = Region[ {DomainCC, DomainC} ];

  Surf_FixedMVP = Region[{ Surf_Inf }];

}

Function {
  nu [ Region[{Air, AirInf, Inds}] ]  = 1./mu0 ;

  nu [Core]  = 1/(mur_fe*mu0) ;

  sigma[Inds] = sigma_coil ;
  sigma[Core] = sigma_core ;
}


// --------------------------------------------------------------------------

Jacobian {
  { Name Vol ;
    Case { 
           { Region All ;       Jacobian Vol ; }
    }
  }
  { Name Sur ;
    Case { { Region All ; Jacobian Sur ; }
    }
  }
}

Integration {
  { Name II ;
    Case {
      {
	Type Gauss ;
	Case {
	  { GeoElement Triangle    ; NumberOfPoints  4 ; }
	  { GeoElement Quadrangle  ; NumberOfPoints  4 ; }
	  { GeoElement Tetrahedron ; NumberOfPoints  4 ; }
	  { GeoElement Hexahedron  ; NumberOfPoints  6 ; }
	  { GeoElement Prism       ; NumberOfPoints  21 ; }
	  { GeoElement Line        ; NumberOfPoints  4 ; }
	}
      }
    }
  }
}

// --------------------------------------------------------------------------

Group {
  Surf_a_NoGauge = Region [ {Surf_FixedMVP, SkinDomainC} ] ;
}

Constraint {

  { Name GaugeCondition_a ; Type Assign ;
    Case {
      { Region DomainCC ; SubRegion Surf_a_NoGauge ; Value 0. ; }
    }
  }
  // av - formulation
  { Name MVP_3D ;
    Case {
      { Region Surf_Inf ; Type Assign ; Value 0. ; }
    }
  }
  { Name Pot ;
    Case {
      { Region Surf_Inf ; Type Assign ; Value 0. ; }
      { Region SkinDomainC ; Type Assign ; Value 0. ; }
    }
  }
}

FunctionSpace {

  // Magnetic vector potential a (b = curl a)
  { Name Hcurl_a_3D ; Type Form1 ;
    BasisFunction {// a = a_e * s_e
      { Name se ; NameOfCoef ae ; Function BF_Edge ;
        Support Domain ; Entity EdgesOf[ All, Not SkinDomainC ] ; }
      { Name se2 ; NameOfCoef ae2 ; Function BF_Edge ;
        Support Domain ; Entity EdgesOf[ SkinDomainC ] ; }
    }
    Constraint {
      { NameOfCoef ae  ; EntityType EdgesOf ; NameOfConstraint MVP_3D ; }
      { NameOfCoef ae2 ; EntityType EdgesOf ; NameOfConstraint MVP_3D ; }
      { NameOfCoef ae  ; EntityType EdgesOfTreeIn ; EntitySubType StartingOn ;
	NameOfConstraint GaugeCondition_a ; }
    }
  }

  // Electric scalar potential (3D)
  { Name Hregion_u_3D ; Type Form0 ;
    BasisFunction {
      { Name sr ; NameOfCoef ur ; Function BF_Node ;
        Support Domain ; Entity NodesOf[ All ] ; }
    }
    Constraint {
      { NameOfCoef ur ; EntityType NodesOf ; NameOfConstraint Pot ; }
    }
  }

}

//---------------------------------------------------------------------------------------------

Formulation {

  { Name MagStaDyn_av_js0_3D ; Type FemEquation ;
    Quantity {
      { Name a  ; Type Local ; NameOfSpace Hcurl_a_3D ; }
      { Name v  ; Type Local ; NameOfSpace Hregion_u_3D ; }
    }

    Equation {
      Galerkin { [ nu[] * Dof{d a} , {d a} ] ;
        In Domain ; Jacobian Vol ; Integration II ; }
      Galerkin { DtDof[ sigma[] * Dof{a} , {a} ] ;
        In DomainC ; Jacobian Vol ; Integration II ; }
      Galerkin { [ sigma[] * Dof{d v} , {a} ] ;
        In DomainC ; Jacobian Vol ; Integration II ; }

      Galerkin { [ -js0[], {a} ] ;
        In  DomainS ; Jacobian Vol ; Integration II ; }


      Galerkin{ DtDof[ sigma[] * Dof{a}, {d v} ] ;
	In DomainC ; Jacobian Vol ; Integration II ; }
      Galerkin{ [ sigma[] * Dof{d v} , {d v} ] ;
	In DomainC ; Jacobian Vol ; Integration II ; }

    }
  }

}


Resolution {

  { Name Analysis ;
    System {
      { Name Sys ; NameOfFormulation MagStaDyn_av_js0_3D ; Type ComplexValue ; Frequency Freq ; }
    }
    Operation {
      CreateDir["ring3d_tg/"] ;

      InitSolution[Sys] ;

      Generate[Sys] ;

      Solve[Sys] ;

      SaveSolution[Sys] ;

      PostOperation[Get_LocalFields] ;
      PostOperation[Get_GlobalQuantities] ;

    }
  }
}

//-----------------------------------------------------------------------------------------------

PostProcessing {

  { Name MagStaDyn_av_js0_3D ; NameOfFormulation MagStaDyn_av_js0_3D ;
    PostQuantity {
      { Name a ; Value { Term { [ {a} ]          ; In Domain ; Jacobian Vol ; } } }
      { Name b ; Value { Term { [ {d a} ]        ; In Domain ; Jacobian Vol ; } } }
      { Name h ; Value { Term { [ nu[{d a}] * {d a} ] ; In Domain ; Jacobian Vol ; } } }

      { Name v ; Value { Term { [ {v} ]          ; In DomainC ; Jacobian Vol ; } } }
      { Name e ; Value { Term { [ -(Dt[{a}]+{d v}) ] ; In DomainC ; Jacobian Vol ; } } }
      { Name j ; Value { Term { [ -sigma[]*(Dt[{a}]+{d v}) ] ; In DomainC ; Jacobian Vol ; } } }
      { Name js ; Value { Term { [ js0[] ]      ; In DomainS ; Jacobian Vol ; } } }
      
      { Name JouleLosses ;
        Value { Integral {
            [ sigma[]*SquNorm[Dt[{a}]+{d v}] ] ;
            In DomainC ; Jacobian Vol ; Integration II ; }
        }
      }
      { Name MagEnergy ;
        Value { Integral {
            [ 1/2 * nu[{d a}]*{d a} * {d a} ] ;
	    In Domain ; Jacobian Vol ; Integration II ; }
	}
      }

      { Name Flux ; Value {
          Integral { [ js0[]*{a} ] ;
            In Inds  ; Jacobian Vol ; Integration II ; }
        }
      }

      { Name Upos ;
        Value { Integral { Type Global ;
            [ -sigma[] * (Dt[{a}] + {d v}) * BF{d v} ] ;
            In DomainC ; Jacobian Vol ; Integration II ;
          }
        }
      }

    }
  }
}

//-----------------------------------------------------------------------------------------------
 PostOperation Get_LocalFields UsingPost MagStaDyn_av_js0_3D {
   Print[ js, OnElementsOf DomainS, File StrCat[Dir, "js", ExtGmsh], LastTimeStepOnly ] ;
   Print[ a, OnElementsOf Domain, File StrCat[Dir, "a", ExtGmsh], LastTimeStepOnly ] ;
   Print[ v, OnElementsOf Domain, File StrCat[Dir,"v",ExtGmsh], LastTimeStepOnly ] ;
   Print[ b, OnElementsOf Domain, File StrCat[Dir,"b",ExtGmsh], LastTimeStepOnly ] ;
   Print[ j, OnElementsOf DomainC, File StrCat[Dir,"j",ExtGmsh], LastTimeStepOnly ] ;
 }

DefineConstant[
  R_ = {"Analysis", Name "GetDP/1ResolutionChoices", Visible 0},
  C_ = {"-solve -v2", Name "GetDP/9ComputeCommand", Visible 0},
  P_ = {"", Name "GetDP/2PostOperationChoices", Visible 0}
];
